package sun.print;

import javax.print.event.PrintServiceAttributeEvent;
import javax.print.attribute.HashPrintServiceAttributeSet;
import javax.print.event.PrintServiceAttributeListener;
import javax.print.attribute.PrintServiceAttributeSet;
import java.util.Vector;
import javax.print.PrintService;

class ServiceNotifier extends Thread
{
    private PrintService service;
    private Vector listeners;
    private boolean stop;
    private PrintServiceAttributeSet lastSet;
    
    ServiceNotifier(PrintService service) {
        super(service.getName() + " notifier");
        this.stop = false;
        this.service = service;
        this.listeners = new Vector();
        try {
            this.setPriority(4);
            this.setDaemon(true);
            this.start();
        }
        catch (SecurityException ex) {}
    }
    
    void addListener(PrintServiceAttributeListener e) {
        synchronized (this) {
            if (e == null || this.listeners == null) {
                return;
            }
            this.listeners.add(e);
        }
    }
    
    void removeListener(PrintServiceAttributeListener o) {
        synchronized (this) {
            if (o == null || this.listeners == null) {
                return;
            }
            this.listeners.remove(o);
        }
    }
    
    boolean isEmpty() {
        return this.listeners == null || this.listeners.isEmpty();
    }
    
    void stopNotifier() {
        this.stop = true;
    }
    
    void wake() {
        try {
            this.interrupt();
        }
        catch (SecurityException ex) {}
    }
    
    @Override
    public void run() {
        long n = 15000L;
        long millis = 2000L;
        while (!this.stop) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {}
            synchronized (this) {
                if (this.listeners != null) {
                    long currentTimeMillis = System.currentTimeMillis();
                    if (this.listeners != null) {
                        PrintServiceAttributeSet attributes;
                        if ((this.service instanceof AttributeUpdater)) {
                            attributes = ((AttributeUpdater)this.service).getUpdatedAttributes();
                        }
                        else {
                            attributes = this.service.getAttributes();
                        }
                        if (attributes != null && !attributes.isEmpty()) {
                            for (int i = 0; i < this.listeners.size(); ++i) {
                                ((PrintServiceAttributeListener)this.listeners.elementAt(i)).attributeUpdate(new PrintServiceAttributeEvent(this.service, new HashPrintServiceAttributeSet(attributes)));
                            }
                        }
                    }
                    millis = (System.currentTimeMillis() - currentTimeMillis) * 10L;
                    if (millis < n) {
                        millis = n;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
}
